/**
 * Copyright (c) 2020 ~ 2021 KylinSec Co., Ltd. 
 * kiran-cpanel-menu is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2. 
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2 
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND, 
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT, 
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.  
 * See the Mulan PSL v2 for more details.  
 * 
 * Author:     liuxinhao <liuxinhao@kylinos.com.cn>
 */
#include "kiran-start-menu-display-label.h"
#include <QPainter>

KiranStartMenuDisplayLabel::KiranStartMenuDisplayLabel(QWidget *parent) : QLabel(parent), m_bgOpacity(0)
{
    m_backgroundImage.load(":/kcp-menu-images/default.jpg");
}

void KiranStartMenuDisplayLabel::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::HighQualityAntialiasing);

    if (!m_backgroundImage.isNull())
    {
        painter.drawImage(this->rect(), m_backgroundImage);
    }

    float w_2 = 0.5 * this->width();
    float w_2_10 = 0.1 * w_2;
    float w_2_10_10 = 0.1 * w_2_10;
    float h_4 = 0.25 * this->height();

    QRectF r1(w_2_10_10 * 2, h_4, w_2_10, h_4 * 3);
    QRectF r2(w_2_10 + w_2_10_10 * 5, h_4, w_2_10 * 4, h_4 * 3);
    QRectF r3(w_2 / 2 + w_2_10_10 * 7, h_4, w_2_10 * 5, h_4 * 3);

    //r1
    painter.setPen(Qt::black);
    painter.setBrush(Qt::black);
    painter.setOpacity(m_bgOpacity);
    painter.drawRect(r1);
    drawR1Items(r1, &painter);
    //r2
    painter.drawRect(r2);
    drawR2Items(r2, &painter);
    //r3
    if ("expand" == m_displayMd)
    {
        painter.drawRect(r3);
        drawR3Items(r3, &painter);
    }

    QLabel::paintEvent(event);
}

void KiranStartMenuDisplayLabel::drawR1Items(QRectF rectf, QPainter *painter)
{
    if (Q_UNLIKELY(nullptr == painter)) return;

    float h = rectf.height() / 50;
    float w = rectf.width() / 6;
    painter->save();
    painter->setBrush(QBrush(QColor("#43a3f2")));
    for (int i = 1; i <= 4; ++i)
    {
        painter->drawRect(QRectF(rectf.x() + w, rectf.bottomLeft().y() - (4 * h) * i, w * 3, 3 * h));
    }
    painter->drawRect(QRectF(rectf.x() + w, rectf.y() + h, w * 3, 3 * h));
    painter->restore();
}

void KiranStartMenuDisplayLabel::drawR2Items(QRectF rectf, QPainter *painter)
{
    if (Q_UNLIKELY(nullptr == painter)) return;

    float h = rectf.height() / 29;
    float w = rectf.width() / 15;
    painter->save();
    painter->setBrush(QBrush(QColor("#43a3f2")));
    for (int i = 1; i <= 6; ++i)
    {
        painter->drawRect(QRectF(rectf.x() + w, rectf.y() + 3 * h + (4 * h) * (i - 1), w * 3, 3 * h));
    }
    painter->setBrush(QBrush(QColor("#ffffff")));
    for (int i = 1; i <= 6; ++i)
    {
        painter->drawRect(QRectF(rectf.x() + 5 * w, rectf.y() + (4 * h) * i, w * 9, h));
    }
    painter->restore();
}

void KiranStartMenuDisplayLabel::drawR3Items(QRectF rectf, QPainter *painter)
{
    if (Q_UNLIKELY(nullptr == painter)) return;

    float h = rectf.height() / 27;
    float w = rectf.width() / 13;

    for (int m = 1; m <= 2; m++)
    {
        painter->save();
        painter->setBrush(QBrush(QColor("#43a3f2")));
        for (int i = 1; i <= 2; ++i)
        {
            for (int j = 1; j <= 3; ++j)
            {
                painter->drawRect(QRectF(rectf.x() + w + (j - 1) * 4 * w, rectf.y() + (m - 1) * 12 * h + 5 * h + (4 * h) * (i - 1), w * 3, 3 * h));
            }
        }
        painter->setBrush(QBrush(QColor("#ffffff")));
        painter->drawRect(QRectF(rectf.x() + w, rectf.y() + (m - 1) * 12 * h + 3 * h, w * 4, h));

        if (m == 1)
        {
            painter->setPen(Qt::gray);
            painter->drawLine(QPointF(rectf.x(), rectf.y() + 13 * h), QPointF(rectf.x() + rectf.width(), rectf.y() + 13 * h));
        }
        painter->restore();
    }
}

void KiranStartMenuDisplayLabel::setDefultPge(const QString &defultPge)
{
    m_defultPge = defultPge;
    update();
}

void KiranStartMenuDisplayLabel::setDisplayMd(const QString &displayMd)
{
    m_displayMd = displayMd;
    update();
}

void KiranStartMenuDisplayLabel::setBgOpacity(double bgOpacity)
{
    m_bgOpacity = bgOpacity;
    update();
}
