/**
 * Copyright (c) 2020 ~ 2021 KylinSec Co., Ltd. 
 * kiran-cpanel-menu is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2. 
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2 
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND, 
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT, 
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.  
 * See the Mulan PSL v2 for more details.  
 * 
 * Author:     liuxinhao <liuxinhao@kylinos.com.cn>
 */

#include "kiran-start-menu-settings.h"
#include "ui_kiran-start-menu-settings.h"
#include "start-menu-settings-global.h"
#include "widget-property-helper.h"
#include <QButtonGroup>
#include "qgsettings.h"

KiranStartMenuSettings::KiranStartMenuSettings(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::KiranStartMenuSettings)
{
    ui->setupUi(this);
    Kiran::WidgetPropertyHelper::setSpinboxButtonPosition(ui->spinBox, Kiran::ARROW_TWO_SIDERS);
    m_gsettings = new QGSettings(GSETTING_SCHEMA_ID, QByteArray());

    auto *btnGroup_xsms = new QButtonGroup(this);
    btnGroup_xsms->addButton(ui->radioButton_kzms, 0);
    btnGroup_xsms->addButton(ui->radioButton_jcms, 1);
    connect(btnGroup_xsms, SIGNAL(buttonToggled(int,bool)), this, SLOT(onXSMSButtonToggled(int,bool)));

    auto *btnGroup_mrye = new QButtonGroup(this);
    btnGroup_mrye->addButton(ui->radioButton_scj, 0);
    btnGroup_mrye->addButton(ui->radioButton_yycxlb, 1);
    connect(btnGroup_mrye, SIGNAL(buttonToggled(int,bool)), this, SLOT(onMRYEButtonToggled(int,bool)));

    updateWidget();

    connect(m_gsettings, &QGSettings::changed, this, [=](const QString &){
        btnGroup_xsms->blockSignals(true);
        btnGroup_mrye->blockSignals(true);
        ui->spinBox->blockSignals(true);

        updateWidget();

        btnGroup_xsms->blockSignals(false);
        btnGroup_mrye->blockSignals(false);
        ui->spinBox->blockSignals(false);
    });
}

KiranStartMenuSettings::~KiranStartMenuSettings()
{
    delete ui;
    delete m_gsettings;
}

void KiranStartMenuSettings::onXSMSButtonToggled(int id, bool checked)
{
    if(Q_UNLIKELY(nullptr == m_gsettings)) return;

    ui->label_display->setDisplayMd(ui->radioButton_kzms->isChecked() ? "expand" : "compact");

    if(id == 0)
    {
        ui->label_mrym->setVisible(!checked);
        ui->radioButton_scj->setVisible(!checked);
        ui->radioButton_yycxlb->setVisible(!checked);
    }
    else if(id == 1)
    {
        ui->label_mrym->setVisible(checked);
        ui->radioButton_scj->setVisible(checked);
        ui->radioButton_yycxlb->setVisible(checked);
    }

    onOk();
}

void KiranStartMenuSettings::onMRYEButtonToggled(int , bool )
{
    ui->label_display->setDefultPge(ui->radioButton_scj->isChecked() ? "favorite apps" : "all apps");

    onOk();
}

void KiranStartMenuSettings::onOk()
{
    if(Q_UNLIKELY(nullptr == m_gsettings)) return;

    m_gsettings->set("background-opacity", 0.01*ui->spinBox->value());
    m_gsettings->set("display-mode", ui->radioButton_kzms->isChecked() ? "expand" : "compact");

    if(ui->radioButton_jcms->isChecked())
    {
        m_gsettings->set("default-page", ui->radioButton_scj->isChecked() ? "favorite apps" : "all apps");
    }
}

void KiranStartMenuSettings::onCancel()
{
    if(Q_UNLIKELY( nullptr == parent() ))
    {
        this->close();
    }
    else
    {
        sigClose();
    }
}

void KiranStartMenuSettings::updateWidget()
{
    if(Q_UNLIKELY(nullptr == m_gsettings)) return;

    double bgOpacity = m_gsettings->get("background-opacity").toDouble();
    QString displayMd = m_gsettings->get("display-mode").toString();
    QString defultPge = m_gsettings->get("default-page").toString();

    ui->spinBox->setValue(bgOpacity*100);
    ui->radioButton_kzms->setChecked("expand" == displayMd);
    ui->radioButton_jcms->setChecked("compact" == displayMd);
    ui->radioButton_scj->setChecked("favorite apps" == defultPge);
    ui->radioButton_yycxlb->setChecked("all apps" == defultPge);

    ui->label_display->setBgOpacity(bgOpacity);
    ui->label_display->setDisplayMd(displayMd);
    ui->label_display->setDefultPge(defultPge);
}

void KiranStartMenuSettings::on_spinBox_valueChanged(int arg1)
{
    ui->label_display->setBgOpacity(0.01*arg1);

    onOk();
}

QSize KiranStartMenuSettings::sizeHint() const
{
    return QSize(709,468);
}
